@@ -1,5 +1,5 @@
 
-/*  A Bison parser, made from filt_parser.y
+/*  A Bison parser, made from ./filt_parser.y
     by GNU Bison version 1.28  */
 
 #define YYBISON 1  /* Identify Bison output.  */
@@ -37,7 +37,7 @@
 #define	BOOL	280
 #define	IPADDR	281
 
-#line 1 "filt_parser.y"
+#line 1 "./filt_parser.y"
 
 /*
  * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
@@ -104,7 +104,7 @@
 
 
 
-#line 70 "filt_parser.y"
+#line 70 "./filt_parser.y"
 typedef union	{ /* the types that we use in the tokens */
     char *string;
     long signed_long;
@@ -803,111 +803,111 @@ yyreduce:
   switch (yyn) {
 
 case 1:
-#line 111 "filt_parser.y"
+#line 111 "./filt_parser.y"
 {InstallFilter(yyvsp[-1].pf);;
     break;}
 case 2:
-#line 116 "filt_parser.y"
+#line 116 "./filt_parser.y"
 { yyval.pf = MakeBinaryNode(OP_AND,yyvsp[-2].pf,yyvsp[0].pf);;
     break;}
 case 3:
-#line 118 "filt_parser.y"
+#line 118 "./filt_parser.y"
 { yyval.pf = MakeBinaryNode(OP_OR,yyvsp[-2].pf,yyvsp[0].pf);;
     break;}
 case 4:
-#line 120 "filt_parser.y"
+#line 120 "./filt_parser.y"
 { yyval.pf = MakeUnaryNode(OP_NOT,yyvsp[0].pf); ;
     break;}
 case 5:
-#line 122 "filt_parser.y"
+#line 122 "./filt_parser.y"
 { yyval.pf = MakeBinaryNode(yyvsp[-1].op,yyvsp[-2].pf,yyvsp[0].pf);;
     break;}
 case 6:
-#line 124 "filt_parser.y"
+#line 124 "./filt_parser.y"
 { yyval.pf = yyvsp[0].pf; ;
     break;}
 case 7:
-#line 130 "filt_parser.y"
+#line 130 "./filt_parser.y"
 { yyval.pf = MakeBinaryNode(OP_PLUS,yyvsp[-2].pf,yyvsp[0].pf);;
     break;}
 case 8:
-#line 132 "filt_parser.y"
+#line 132 "./filt_parser.y"
 { yyval.pf = MakeBinaryNode(OP_MINUS,yyvsp[-2].pf,yyvsp[0].pf);;
     break;}
 case 9:
-#line 134 "filt_parser.y"
+#line 134 "./filt_parser.y"
 { yyval.pf = MakeBinaryNode(OP_TIMES,yyvsp[-2].pf,yyvsp[0].pf);;
     break;}
 case 10:
-#line 136 "filt_parser.y"
+#line 136 "./filt_parser.y"
 { yyval.pf = MakeBinaryNode(OP_DIVIDE,yyvsp[-2].pf,yyvsp[0].pf);;
     break;}
 case 11:
-#line 138 "filt_parser.y"
+#line 138 "./filt_parser.y"
 { yyval.pf = MakeBinaryNode(OP_MOD,yyvsp[-2].pf,yyvsp[0].pf);;
     break;}
 case 12:
-#line 140 "filt_parser.y"
+#line 140 "./filt_parser.y"
 { yyval.pf = MakeBinaryNode(OP_BAND,yyvsp[-2].pf,yyvsp[0].pf);;
     break;}
 case 13:
-#line 142 "filt_parser.y"
+#line 142 "./filt_parser.y"
 { yyval.pf = MakeBinaryNode(OP_BOR,yyvsp[-2].pf,yyvsp[0].pf);;
     break;}
 case 14:
-#line 144 "filt_parser.y"
+#line 144 "./filt_parser.y"
 { yyval.pf = yyvsp[-1].pf; ;
     break;}
 case 15:
-#line 146 "filt_parser.y"
+#line 146 "./filt_parser.y"
 { yyval.pf = yyvsp[0].pf; ;
     break;}
 case 16:
-#line 151 "filt_parser.y"
+#line 151 "./filt_parser.y"
 { yyval.pf = MakeVarNode(yyvsp[0].string); ;
     break;}
 case 17:
-#line 153 "filt_parser.y"
+#line 153 "./filt_parser.y"
 { yyval.pf = MakeSignedConstNode(yyvsp[0].signed_long); ;
     break;}
 case 18:
-#line 155 "filt_parser.y"
+#line 155 "./filt_parser.y"
 { yyval.pf = MakeUnsignedConstNode(yyvsp[0].unsigned_long); ;
     break;}
 case 19:
-#line 157 "filt_parser.y"
+#line 157 "./filt_parser.y"
 { yyval.pf = MakeStringConstNode(yyvsp[0].string); ;
     break;}
 case 20:
-#line 159 "filt_parser.y"
+#line 159 "./filt_parser.y"
 { yyval.pf = MakeBoolConstNode(yyvsp[0].bool); ;
     break;}
 case 21:
-#line 162 "filt_parser.y"
+#line 162 "./filt_parser.y"
 { yyval.pf = MakeIPaddrConstNode(yyvsp[0].pipaddr); ;
     break;}
 case 22:
-#line 167 "filt_parser.y"
+#line 167 "./filt_parser.y"
 { yyval.op = OP_GREATER;;
     break;}
 case 23:
-#line 169 "filt_parser.y"
+#line 169 "./filt_parser.y"
 { yyval.op = OP_GREATER_EQ;;
     break;}
 case 24:
-#line 171 "filt_parser.y"
+#line 171 "./filt_parser.y"
 { yyval.op = OP_LESS;;
     break;}
 case 25:
-#line 173 "filt_parser.y"
+#line 173 "./filt_parser.y"
 { yyval.op = OP_LESS_EQ;;
     break;}
 case 26:
-#line 175 "filt_parser.y"
+#line 175 "./filt_parser.y"
 { yyval.op = OP_EQUAL;;
     break;}
 case 27:
-#line 177 "filt_parser.y"
+#line 177 "./filt_parser.y"
 { yyval.op = OP_NEQUAL;;
     break;}
 }
@@ -1132,7 +1132,7 @@ yyerrhandle:
     }
   return 1;
 }
-#line 179 "filt_parser.y"
+#line 179 "./filt_parser.y"
 
 
 void
