@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+ *               2002, 2003, 2004
  *	Ohio University.
  *
  * ---
@@ -50,13 +51,13 @@
  * 		Athens, OH
  *		http://www.tcptrace.org/
  */
-static char const copyright[] =
-    "@(#)Copyright (c) 2001 -- Ohio University.\n";
-static char const rcsid[] =
-    "@(#)$Header: /usr/local/cvs/tcptrace/ipv6.c,v 5.19 2001/12/03 15:58:49 sdo Exp $";
-
-
 #include "tcptrace.h"
+static char const GCC_UNUSED copyright[] =
+    "@(#)Copyright (c) 2004 -- Ohio University.\n";
+static char const GCC_UNUSED rcsid[] =
+    "@(#)$Header: /usr/local/cvs/tcptrace/ipv6.c,v 5.23 2003/11/19 14:38:02 sdo Exp $";
+
+
 
 /* the names of IPv6 extensions that we understand */
 char *
@@ -255,7 +256,7 @@ findheader(
 		  
 	      pheader = (struct ipv6_ext *)
 		  ((char *)pheader + 8 + (pheader->ip6ext_len)*8);
-
+	      break;
 	    /* non-tcp protocols, so we're finished. */
 	  case IPPROTO_NONE:
 	  case IPPROTO_ICMPV6:
@@ -355,14 +356,28 @@ int gethdrlength (struct ip *pip, void *plast)
 		pheader += 8;
 		length += 8;
 	    }
-	    if ((nextheader == IPPROTO_HOPOPTS) || (nextheader == IPPROTO_ROUTING)
-		|| (nextheader == IPPROTO_DSTOPTS))
+	    if ((nextheader == IPPROTO_HOPOPTS) || 
+		(nextheader == IPPROTO_ROUTING) ||
+		(nextheader == IPPROTO_DSTOPTS))
 	    {
-		nextheader = *pheader;
-		pheader += *(pheader + 1);
-		length += *(pheader + 1);
+	      // Thanks to patch sent by Thomas Bohnert
+	      // Header length field in these IPv6 extension headers
+	      // stores the length of the header in units of 8 bytes, 
+	      // *without* counting the mandatory 8 bytes
+	      
+	      nextheader = *pheader;
+	      length += (*(pheader+1) + 1) * 8;
+	      pheader += (*(pheader+1) + 1) * 8;
 	    }
-	    if (pheader > (char *)plast)
+	    // IPv6 encapsulated in IPv6
+	    if (nextheader == IPPROTO_IPV6)
+	    {
+	      pheader += 40;
+	      nextheader=*(pheader+6);
+	      length += 40;
+	    }
+
+	  if (pheader > (char *)plast)
 		return -1;
 	}
     }
@@ -418,7 +433,7 @@ int IP_SAMEADDR (ipaddr addr1, ipaddr addr2)
 	if (ADDR_ISV4(&addr2))
 	    ret = (addr1.un.ip4.s_addr == addr2.un.ip4.s_addr);
     }
-    if (debug > 3)
+   if (debug > 3)
 	printf("SameAddr(%s(%d),%s(%d)) returns %d\n",
 	       HostName(addr1), ADDR_VERSION(&addr1),
 	       HostName(addr2), ADDR_VERSION(&addr2),
@@ -426,6 +441,28 @@ int IP_SAMEADDR (ipaddr addr1, ipaddr addr2)
     return ret;
 }
 
+/*  
+ *  iplowaddr: test if one IPv4 or IPv6 address is lower than the second one
+ */
+int IP_LOWADDR (ipaddr addr1, ipaddr addr2)
+{
+   int ret = 0;
+   if (ADDR_ISV6(&addr1)) {
+      if (ADDR_ISV6(&addr2))
+	ret = (memcmp(addr1.un.ip6.s6_addr,
+		      addr2.un.ip6.s6_addr,16) < 0);
+   } else {
+      if (ADDR_ISV4(&addr2))
+	ret = (addr1.un.ip4.s_addr < addr2.un.ip4.s_addr);
+   }
+   
+   if (debug > 3)
+     printf("LowAddr(%s(%d),%s(%d)) returns %d\n",
+	    HostName(addr1), ADDR_VERSION(&addr1),
+	    HostName(addr2), ADDR_VERSION(&addr2),
+	    ret);
+   return ret;
+}
 
 
 #ifndef HAVE_INET_PTON
