@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+ *               2002, 2003, 2004
  *	Ohio University.
  *
  * ---
@@ -51,8 +52,22 @@
  *		ostermann@cs.ohiou.edu
  *		http://www.tcptrace.org/
  */
-static char const rcsid_tcptrace[] =
-    "@(#)$Header: /usr/local/cvs/tcptrace/tcptrace.h,v 5.57 2002/07/26 07:22:49 alakhian Exp $";
+
+
+/*
+ *
+ * The newest gcc (3.3) is much pickier about the unused variables that
+ * we define for Copyright and RCSid, so this will shut it up.
+ *
+ */
+#ifdef __GNUC__
+#define GCC_UNUSED __attribute__((unused))
+#else
+#define GCC_UNUSED
+#endif
+
+static char const GCC_UNUSED rcsid_tcptrace[] =
+    "@(#)$Header: /usr/local/cvs/tcptrace/tcptrace.h,v 5.75 2003/11/19 20:13:35 ryellapr Exp $";
 
 
 #include <stdio.h>
@@ -131,7 +146,6 @@ typedef struct pl_line *PLINE;
 #define MAX_HOSTLETTER_LEN 8 
 
 
-
 /* several places in the code NEED numbers of a specific size. */
 /* since the definitions aren't standard across everything we're */
 /* trying to support, the types are gathered up here */
@@ -253,6 +267,8 @@ extern u_long udp_trace_count;
 #ifdef OLD
 /* test 2 IP addresses for equality */
 #define IP_SAMEADDR(addr1,addr2) (((addr1).s_addr) == ((addr2).s_addr))
+/* test for an IP address lower than the second IP address */
+#define IP_LOWADDR(addr1,addr2) (((addr1).s_addr) < ((addr2).s_addr))
 
 /* copy IP addresses */
 #define IP_COPYADDR(toaddr,fromaddr) ((toaddr).s_addr = (fromaddr).s_addr)
@@ -303,7 +319,8 @@ typedef struct tcb {
 	/* If we are using window scaling, have we adjusted the 
 	   win_min field from the non-scaled window size
 	   that appeared in the SYN packet?? */
-	Bool window_stats_updated_for_scaling;
+    Bool window_stats_updated_for_scaling;
+    u_llong     win_scaled_pkts; /* Used to calculate avg win adv */
 
     /* statistics added */
     u_llong	data_bytes;
@@ -316,7 +333,7 @@ typedef struct tcb {
     u_llong	pureack_pkts;	/* mallman - pure acks, no data */
     u_long	win_max;
     u_long	win_min;
-    u_long	win_tot;
+    u_llong	win_tot;
     u_long      win_last;  /* last advertised window size*/
     u_long	win_zero_ct;
     u_llong	packets;
@@ -334,9 +351,6 @@ typedef struct tcb {
     u_long     urg_data_bytes;
     u_long     urg_data_pkts;
    
-   /* stats on sequence numbers */
-
-								
    /* Statistics to store the number of Zero window probes
       seen and the total number of bytes spent for it. */
     u_long      num_zwnd_probes;  
@@ -472,6 +486,7 @@ typedef struct tcb {
     /* Congestion window graph */
     PLOTTER	owin_plotter;
     PLINE	owin_line;
+    PLINE       rwin_line;
     PLINE	owin_avg_line;
     PLINE 	owin_wavg_line;
 
@@ -482,6 +497,14 @@ typedef struct tcb {
     /* for looking for interesting SACK blocks */
     u_long	num_sacks;
     u_long	max_sack_blocks;
+    u_long	num_dsacks;
+
+    /* for computing LEAST (see FAQ) */
+    enum	tcp_strains { TCP_RENO, TCP_SACK, TCP_DSACK } tcp_strain;
+    u_long	LEAST;
+    char	in_rto;
+    u_long	recovered, recovered_orig, rto_segment, lastackno;
+    u_long	event_retrans, event_dupacks;
 
     /* host name letter(s) */
     char	*host_letter;
@@ -539,6 +562,38 @@ typedef struct tcphdr tcphdr;
 extern int num_tcp_pairs;	/* how many pairs are in use */
 extern tcp_pair **ttp;		/* array of pointers to allocated pairs */
 
+/* Wed Aug 20, 2003 - Ramani*/
+/*  Prior to version 6.4.11, the data structure for storing the snapshots of 
+ connections was a hashtable with linked lists. But this might lead to a 
+ worst case scenario when many connections hash to the same hash table entry.
+ In such a case, searching for the connections degrades to searching a linked 
+ list with a worst case complexity of O(number of connections in list). Hence 
+ the new version implements an AVL tree in place of linked list leading to a 
+ worst case complexity of O(ln(number of connections in tree)). 
+    The modified data structure was tested with dumpfiles containing lots of 
+ connections. A comparison of the profiles suggests an improvement in the time 
+ spent in the dotrace function. Even though the AVL tree implementation 
+ involves balancing the tree, since most of the accesses involve searching the 
+ data structure, AVL tree performs MUCH better than linked list.
+     The algorithms for AVL tree implementation are based on those explained in
+ "Data Structures and Program Design in C by Robert L.Kruse, Bruce P.Leung, 
+ Clovis L.Tondo". The source code for AVL tree implementation is from the 
+ Institute of Applied Iconoclasm who put up the source code at 
+ <http://www.purists.org>. We thank Georg for the source code whose mail 
+ address has been mentioned as <georg@purists.org> */
+
+/* Data structures for AVL tree */
+
+/* Which of a given node's subtrees is higher in the AVL tree */
+enum AVLSKEW {
+   EQUAL1, LEFT, RIGHT
+};
+                                                                                
+/* Did an insertion/deletion succeed and if we need to balance the AVL tree */
+enum AVLRES {
+   OK, BALANCE
+};
+
 /* Tue Nov 17, 1998 */
 /* prior to version 5.13, we kept a hash table of all of the connections. */
 /* The most recently-accessed connections move to the front of the bucket */
@@ -550,11 +605,13 @@ extern tcp_pair **ttp;		/* array of pointers to allocated pairs */
 /* hash table.  We only retrieve the connection record if the snapshot */
 /* matches. The result is that it works MUCH better when memory is low. */
 typedef struct ptp_snap {
+    enum AVLSKEW        skew;      /* Skew of the AVL tree node */
     tcp_pair_addrblock	addr_pair; /* just a copy */
-    struct ptp_snap	*next;
+    struct ptp_snap     *left, *right;  /* Left and right trees of the AVL node */
     void		*ptp;
 } ptp_snap;
 
+
 typedef struct ptp_ptr {
   struct ptp_ptr	*next;
   struct ptp_ptr	*prev;
@@ -562,6 +619,7 @@ typedef struct ptp_ptr {
   tcp_pair		*ptp;
 } ptp_ptr;
 
+#define NONREAL_LIVE_CONN_INTERVAL      4*60    /* 4 minutes */
 #define REMOVE_LIVE_CONN_INTERVAL	8*3600	/* 8 hours */
 #define REMOVE_CLOSED_CONN_INTERVAL	8*60	/* 8 minutes */
 #define UPDATE_INTERVAL			30	/* 30 seconds */
@@ -588,6 +646,9 @@ typedef struct ucb {
 
 typedef tcp_pair_addrblock udp_pair_addrblock;
 struct sudp_pair {
+    /* Are we ignoring this 'connection' ? */
+    Bool                ignore_pair;
+     
     /* endpoint identification */
     udp_pair_addrblock	addr_pair;
 
@@ -669,13 +730,18 @@ extern Bool plot_tput_instant;
 extern Bool filter_output;
 extern Bool do_udp;
 extern Bool show_title;
+extern Bool show_rwinline;
 extern Bool docheck_hw_dups;
 /* constants for real-time (continuous) mode */
 extern Bool run_continuously;
 extern Bool conn_num_threshold;
 extern Bool xplot_all_files;
 extern Bool ns_hdrs;
+extern Bool dup_ack_handling;
+extern Bool csv;
+extern Bool tsv;
 extern u_long remove_live_conn_interval;
+extern u_long nonreal_live_conn_interval;
 extern u_long remove_closed_conn_interval;
 extern u_long update_interval;
 extern u_long max_conn_num;
@@ -689,6 +755,13 @@ extern char *output_file_dir;
 extern char *output_file_prefix;
 extern char *xplot_title_prefix;
 extern char *xplot_args;
+extern char *sv;
+extern char *sp;       /* Separator used for long output with <SP>-separated-values */
+
+/* Used to comment out header lines of the long output
+ * when <SP>-separated-values is requested
+ */
+extern char *comment;
 
 extern u_long ctrunc;
 extern timeval current_time;
@@ -751,10 +824,13 @@ void PrintRawData(char *label, void *pfirst, void *plast, Bool octal);
 void PrintRawDataHex(char *label, void *pfirst, void *plast);
 void PrintTrace(tcp_pair *);
 void UDPPrintTrace(udp_pair *);
+void PrintSVHeader(void);
 void PrintBrief(tcp_pair *);
 void UDPPrintBrief(udp_pair *);
 void OnlyConn(int);
 void IgnoreConn(int);
+void OnlyUDPConn(int);
+void IgnoreUDPConn(int);
 double elapsed(timeval, timeval);
 void tv_sub(struct timeval *plhs, struct timeval rhs);
 void tv_add(struct timeval *plhs, struct timeval rhs);
@@ -776,7 +852,8 @@ char *EndpointName(ipaddr,portnum);
 PLOTTER new_plotter(tcb *plast, char *filename, char *title,
 		    char *xlabel, char *ylabel, char *suffix);
 int rexmit(tcb *, seqnum, seglen, Bool *);
-enum t_ack ack_in(tcb *, seqnum, unsigned tcp_data_length);
+enum t_ack ack_in(tcb *, seqnum, unsigned tcp_data_length, u_long eff_win);
+Bool IsRTO(tcb *ptcb, seqnum s);
 void DoThru(tcb *ptcb, int nbytes);
 struct mfile *Mfopen(char *fname, char *mode);
 void Minit(void);
@@ -799,6 +876,7 @@ Bool FileIsStdin(char *filename);
 struct tcb *ptp2ptcb(tcp_pair *ptp, struct ip *pip, struct tcphdr *ptcp);
 void IP_COPYADDR (ipaddr *toaddr, ipaddr fromaddr);
 int IP_SAMEADDR (ipaddr addr1, ipaddr addr2);
+int IP_LOWADDR (ipaddr addr1, ipaddr addr2);
 void PcapSavePacket(char *filename, struct ip *pip, void *plast);
 void StringToArgv(char *buf, int *pargc, char ***pargv);
 void CopyAddr(tcp_pair_addrblock *, struct ip *pip,portnum,portnum);
@@ -814,6 +892,8 @@ void ModulesPerOldConn(tcp_pair *ptp);
 /* Memory allocation routines with page boundaries */ 
 tcp_pair *MakeTcpPair(void);
 void FreeTcpPair(tcp_pair *ptr);
+udp_pair *MakeUdpPair(void);
+void FreeUdpPair(udp_pair *ptr);
 seqspace *MakeSeqspace(void);
 void FreeSeqspace(seqspace *ptr);
 ptp_snap *MakePtpSnap(void);
@@ -826,6 +906,13 @@ ptp_ptr *MakePtpPtr(void);
 void FreePtpPtr(ptp_ptr *ptr);
 void freequad(quadrant **);
 
+/* AVL tree support routines */
+enum AVLRES SnapInsert(ptp_snap **n, ptp_snap *new_node);
+enum AVLRES SnapRemove(ptp_snap **n, tcp_pair_addrblock address);
+int AVL_CheckHash(tcp_pair_addrblock *ptpa1,
+		  tcp_pair_addrblock *ptpa2, int *pdir);
+int AVL_CheckDir(tcp_pair_addrblock *ptpa1, tcp_pair_addrblock *ptpa2);
+    
 /* high-level line drawing */
 PLINE new_line(PLOTTER pl, char *label, char *color);
 void extend_line(PLINE pline, timeval xval, int yval);
@@ -868,6 +955,9 @@ of bits as specified in RFC 2481 and draft-ietf-tsvwg-ecn-04.txt */
 #define A2B 1
 #define B2A -1
 
+/* If the AVL node is to left or right in the AVL tree */
+#define LOW 2
+#define HIGH 3
 
 /*macros for maintaining the seqspace used for rexmit*/
 #define QUADSIZE	(0x40000000)
@@ -1029,6 +1119,9 @@ typedef int pread_f(struct timeval *, int *, int *, void **,
 #ifdef GROK_NETSCOUT
 	pread_f *is_netscout(char *);
 #endif /* GROK_NETSCOUT */
+#ifdef GROK_ERF
+	pread_f *is_erf(char *);
+#endif /* GROK_ERF */
 
 #ifndef __VMS
 /* I've had problems with the memcpy function that gcc stuffs into the program
@@ -1091,8 +1184,21 @@ struct ipaddr *IPV6ADDR2ADDR(struct in6_addr *addr6);
 #endif /* IP_MAXPACKET */
 
 /* max 32 bit number */
-#define MAX_32 (0x100000000)
+#define MAX_32 (0x100000000LL)
 
 #ifndef ETHERTYPE_REVARP
 #define ETHERTYPE_REVARP        0x8035
 #endif /* ETHERTYPE_REVARP */
+
+#ifndef ETHERTYPE_VLAN
+#define ETHERTYPE_VLAN		0x8100
+#endif	/* 802.1Q Virtual LAN */
+
+/* support for PPPoE encapsulation added by Yann Samama (ysamama@nortelnetworks.com)*/
+#ifndef ETHERTYPE_PPPOE_SESSION
+#define ETHERTYPE_PPPOE_SESSION	0x8864
+#endif /* PPPoE ether type */
+#ifndef PPPOE_SIZE
+#define PPPOE_SIZE		22
+#endif /* PPPOE header size */
+
